@@ -157,8 +157,6 @@ Bits CPU_Core_Normal_Run(void) {
 	if (CPU_Cycles <= 0)
 		return CBRET_NONE;
 
-	const Bitu init_cycles = CPU_Cycles;
-
 	while (CPU_Cycles-->0) {
 		LOADIP;
 		last_prefix=MP_NONE;
@@ -211,9 +209,8 @@ restart_opcode:
 		}
 		SAVEIP;
 	}
-decode_stop_at_instruction:
-	FillFlags();
-	return CBRET_NONE;
+    FillFlags();
+    return CBRET_NONE;
 decode_end:
 	SAVEIP;
 	FillFlags();
