@@ -68,6 +68,7 @@ MidiHandler::MidiHandler(){
 	handler_list = this;
 }
 
+bool roland_gs_sysex = true;
 MidiHandler Midi_none;
 DB_Midi midi;
 std::string sffile="Not available";
@@ -533,6 +534,27 @@ void MIDI_RawOutByte(uint8_t data) {
 			} else {
 //				LOG(LOG_ALL,LOG_NORMAL)("Play sysex; address:%02X %02X %02X, length:%4d, delay:%3d", midi.sysex.buf[5], midi.sysex.buf[6], midi.sysex.buf[7], midi.sysex.used, midi.sysex.delay);
 				midi.handler->PlaySysex(midi.sysex.buf, midi.sysex.used);
+
+				if (roland_gs_sysex) {
+					if (midi.sysex.buf[1] == 0x41/*Roland*/ && midi.sysex.buf[3] == 0x42/*GS*/ && midi.sysex.buf[4] == 0x12/*Send*/) {
+						const uint32_t addr =
+							((uint32_t)midi.sysex.buf[5] << 16) +
+							((uint32_t)midi.sysex.buf[6] <<  8) +
+							(uint32_t)midi.sysex.buf[7];
+
+						switch (addr) {
+							case 0x40007F: /* GS reset */
+								{
+									uint8_t msg[] = {0xFF};
+									midi.handler->PlayMsg(msg); /* MIDI reset */
+								}
+								break;
+							default:
+								break;
+						};
+					}
+				}
+
 				if (midi.sysex.start) {
 					if (midi.sysex.buf[5] == 0x7F) {
 						midi.sysex.delay = 290; // All Parameters reset
@@ -600,6 +622,8 @@ public:
 		MidiHandler * handler;
 		bool opened = false;
 
+		roland_gs_sysex = section->Get_bool("roland gs sysex");
+
 //		MAPPER_AddHandler(MIDI_SaveRawEvent,MK_f8,MMOD1|MMOD2,"caprawmidi","Cap MIDI");
 		midi.sysex.delay = 0;
 		midi.sysex.start = 0;
