@@ -32,6 +32,7 @@
 #include "bitop.h"
 
 int hack_lfb_yadjust = 0;
+int hack_lfb_xadjust = 0;
 
 int vesa_set_display_vsync_wait = -1;
 bool vesa_bank_switch_window_range_check = true;
@@ -252,6 +253,7 @@ uint8_t VESA_GetSVGAModeInformation(uint16_t mode,uint16_t seg,uint16_t off) {
 	PhysPt buf=PhysMake(seg,off);
 	Bitu pageSize;
 	uint8_t modeAttributes;
+	unsigned int adj = 0;
 	Bitu i=0;
 
 	mode&=0x3fff;	// vbe2 compatible, ignore lfb and keep screen content bits
@@ -292,11 +294,14 @@ foundit:
 	bool allow_res = allow_vesa_lowres_modes ||
 		(ModeList_VGA[i].swidth >= 640 && ModeList_VGA[i].sheight >= 400);
 
+	unsigned int cwidth = (mblock->pitch != 0) ? mblock->pitch : mblock->swidth;
+
 	switch (mblock->type) {
 	case M_PACKED4:
 		if (!allow_vesa_4bpp_packed && !(ModeList_VGA[i].mode >= 0x202 && ModeList_VGA[i].mode <= 0x208)) return VESA_FAIL;//TODO: New option to disable
-		pageSize = mblock->sheight * mblock->swidth/2;
-		var_write(&minfo.BytesPerScanLine,(uint16_t)((((mblock->swidth+15U)/8U)&(~1U))*4)); /* NTS: 4bpp requires even value due to VGA registers, round up */
+		pageSize = mblock->sheight * cwidth/2;
+		adj = hack_lfb_xadjust / 2;
+		var_write(&minfo.BytesPerScanLine,(uint16_t)((((cwidth+15U)/8U)&(~1U))*4)); /* NTS: 4bpp requires even value due to VGA registers, round up */
 		if (!int10.vesa_oldvbe10) { /* optional in VBE 1.0 */
 			var_write(&minfo.NumberOfPlanes,0x1);
 			var_write(&minfo.BitsPerPixel,4);
@@ -307,8 +312,9 @@ foundit:
 		break;
 	case M_LIN4:
 		if (!allow_vesa_4bpp) return VESA_FAIL;
-		pageSize = mblock->sheight * (uint16_t)(((mblock->swidth+15U)/8U)&(~1U));
-		var_write(&minfo.BytesPerScanLine,(uint16_t)(((mblock->swidth+15U)/8U)&(~1U))); /* NTS: 4bpp requires even value due to VGA registers, round up */
+		pageSize = mblock->sheight * (uint16_t)(((cwidth+15U)/8U)&(~1U));
+		adj = hack_lfb_xadjust / 8;
+		var_write(&minfo.BytesPerScanLine,(uint16_t)(((cwidth+15U)/8U)&(~1U))); /* NTS: 4bpp requires even value due to VGA registers, round up */
 		if (!int10.vesa_oldvbe10) { /* optional in VBE 1.0 */
 			var_write(&minfo.NumberOfPlanes,0x4);
 			var_write(&minfo.BitsPerPixel,4);   // bits per pixel is 4 as specified by VESA BIOS 2.0 specification
@@ -318,8 +324,9 @@ foundit:
 		break;
 	case M_LIN8:
 		if (!allow_vesa_8bpp || !allow_res) return VESA_FAIL;
-		pageSize = mblock->sheight * mblock->swidth;
-		var_write(&minfo.BytesPerScanLine,(uint16_t)mblock->swidth);
+		pageSize = mblock->sheight * cwidth;
+		adj = hack_lfb_xadjust;
+		var_write(&minfo.BytesPerScanLine,(uint16_t)cwidth);
 		if (!int10.vesa_oldvbe10) { /* optional in VBE 1.0 */
 			var_write(&minfo.NumberOfPlanes,0x1);
 			var_write(&minfo.BitsPerPixel,8);
@@ -330,8 +337,9 @@ foundit:
 		break;
 	case M_LIN15:
 		if (!allow_vesa_15bpp || !allow_res) return VESA_FAIL;
-		pageSize = mblock->sheight * mblock->swidth*2;
-		var_write(&minfo.BytesPerScanLine,(uint16_t)(mblock->swidth*2));
+		pageSize = mblock->sheight * cwidth*2;
+		adj = hack_lfb_xadjust * 2;
+		var_write(&minfo.BytesPerScanLine,(uint16_t)(cwidth*2));
 		if (!int10.vesa_oldvbe10) { /* optional in VBE 1.0 */
 			var_write(&minfo.NumberOfPlanes,0x1);
 			var_write(&minfo.BitsPerPixel,15);
@@ -350,8 +358,9 @@ foundit:
 		break;
 	case M_LIN16:
 		if (!allow_vesa_16bpp || !allow_res) return VESA_FAIL;
-		pageSize = mblock->sheight * mblock->swidth*2;
-		var_write(&minfo.BytesPerScanLine,(uint16_t)(mblock->swidth*2));
+		pageSize = mblock->sheight * cwidth*2;
+		adj = hack_lfb_xadjust * 2;
+		var_write(&minfo.BytesPerScanLine,(uint16_t)(cwidth*2));
 		if (!int10.vesa_oldvbe10) { /* optional in VBE 1.0 */
 			var_write(&minfo.NumberOfPlanes,0x1);
 			var_write(&minfo.BitsPerPixel,16);
@@ -369,8 +378,9 @@ foundit:
 	case M_LIN24:
 		if (!allow_vesa_24bpp || !allow_res) return VESA_FAIL;
 		if (mode >= 0x120 && !allow_explicit_vesa_24bpp) return VESA_FAIL;
-		pageSize = mblock->sheight * mblock->swidth*3;
-		var_write(&minfo.BytesPerScanLine,(uint16_t)(mblock->swidth*3));
+		pageSize = mblock->sheight * cwidth*3;
+		adj = hack_lfb_xadjust * 3;
+		var_write(&minfo.BytesPerScanLine,(uint16_t)(cwidth*3));
 		if (!int10.vesa_oldvbe10) { /* optional in VBE 1.0 */
 			var_write(&minfo.NumberOfPlanes,0x1);
 			var_write(&minfo.BitsPerPixel,24);
@@ -387,8 +397,9 @@ foundit:
 		break;
 	case M_LIN32:
 		if (!allow_vesa_32bpp || !allow_res) return VESA_FAIL;
-		pageSize = mblock->sheight * mblock->swidth*4;
-		var_write(&minfo.BytesPerScanLine,(uint16_t)(mblock->swidth*4));
+		pageSize = mblock->sheight * cwidth*4;
+		adj = hack_lfb_xadjust * 4;
+		var_write(&minfo.BytesPerScanLine,(uint16_t)(cwidth*4));
 		if (!int10.vesa_oldvbe10) { /* optional in VBE 1.0 */
 			var_write(&minfo.NumberOfPlanes,0x1);
 			var_write(&minfo.BitsPerPixel,32);
@@ -407,10 +418,11 @@ foundit:
 		break;
 	case M_TEXT:
 		if (!allow_vesa_tty) return VESA_FAIL;
-		pageSize = 0;
+		adj = hack_lfb_xadjust / 8;
+		pageSize = mblock->sheight * cwidth/8;
 		var_write(&minfo.BytesPerScanLine, (uint16_t)(mblock->twidth * 2));
 		if (!int10.vesa_oldvbe10) { /* optional in VBE 1.0 */
-			var_write(&minfo.NumberOfPlanes,0x4);
+			var_write(&minfo.NumberOfPlanes,0x1);
 			var_write(&minfo.BitsPerPixel,4);
 			var_write(&minfo.MemoryModel,0);	// text
 		}
@@ -426,11 +438,14 @@ foundit:
 		pageSize &= ~0xFFFFu;
 	}
 	Bitu pages = 0;
-	if (pageSize > GetReportedVideoMemorySize() || (mblock->special & _USER_DISABLED)) {
+	Bitu calcmemsize = GetReportedVideoMemorySize();
+	if (mblock->type == M_LIN4) calcmemsize /= 4u; /* 4bpp planar = 4 bytes per video memory byte */
+	if (pageSize > calcmemsize || (mblock->special & _USER_DISABLED)) {
 		// mode not supported by current hardware configuration
 		modeAttributes &= ~0x1;
 	} else if (pageSize) {
-		pages = (GetReportedVideoMemorySize() / pageSize)-1;
+		pages = (calcmemsize / pageSize) - 1;
+		if (pages > 254) pages = 254;
 	}
 
 	/* VBE 1.0 allows fields "XResolution" and later to be optional.
@@ -492,7 +507,8 @@ foundit:
 		}
 	}
 
-	if (!int10.vesa_nolfb && !int10.vesa_oldvbe) var_write(&minfo.PhysBasePtr,S3_LFB_BASE + (hack_lfb_yadjust*(long)host_readw((HostPt)(&minfo.BytesPerScanLine))));
+	if (!int10.vesa_nolfb && !int10.vesa_oldvbe && (modeAttributes&0x80)/*ModeAttributes indicates an LFB*/)
+		var_write(&minfo.PhysBasePtr,S3_LFB_BASE + adj + (hack_lfb_yadjust*(long)host_readw((HostPt)(&minfo.BytesPerScanLine))));
 
 	MEM_BlockWrite(buf,&minfo,sizeof(MODE_INFO));
 	return VESA_SUCCESS;
@@ -527,7 +543,10 @@ uint8_t VESA_SetCPUWindow(uint8_t window,uint16_t address) {
 	 * parameter. */
 	address &= 0xFFu;
 
-	if ((!vesa_bank_switch_window_range_check) || (uint32_t)(address)*vga.svga.bank_size<GetReportedVideoMemorySize()) { /* range check, or silently truncate address depending on dosbox-x.conf setting */
+	Bitu calcmemsize = GetReportedVideoMemorySize();
+	if (CurMode->type == M_LIN4) calcmemsize /= 4u; /* 4bpp planar = 4 bytes per video memory byte */
+
+	if ((!vesa_bank_switch_window_range_check) || (uint32_t)(address)*vga.svga.bank_size<calcmemsize) { /* range check, or silently truncate address depending on dosbox-x.conf setting */
 		IO_Write(0x3d4,0x6a);
 		IO_Write(0x3d5,(uint8_t)address); /* NTS: in vga_s3.cpp this is a 7-bit field, wraparound will occur at address >= 128 but only if emulating a full 64KB bank as normal */
 		return VESA_SUCCESS;
@@ -621,7 +640,8 @@ uint8_t VESA_ScanLineLength(uint8_t subcall,uint16_t val, uint16_t & bytes,uint1
 			pixels_per_offset = 4;
 			break;
 		case M_LIN24:
-			pixels_per_offset = 2;
+			pixels_per_offset = 8;
+			bytes_per_offset = 24;
 			break;
 		case M_LIN32:
 			pixels_per_offset = 2;
@@ -651,7 +671,11 @@ uint8_t VESA_ScanLineLength(uint8_t subcall,uint16_t val, uint16_t & bytes,uint1
 			// TODO: Add dosbox-x.conf option to control which behavior is emulated.
 			if (new_offset > max_offset) new_offset = max_offset;
 
-			vga.config.scan_len = new_offset;
+			if (CurMode->type == M_LIN24)
+				vga.config.scan_len = new_offset * 3u;
+			else
+				vga.config.scan_len = new_offset;
+
 			VGA_CheckScanLength();
 			break;
 
@@ -672,7 +696,11 @@ uint8_t VESA_ScanLineLength(uint8_t subcall,uint16_t val, uint16_t & bytes,uint1
 			// TODO: Add dosbox-x.conf option to control which behavior is emulated.
 			if (new_offset > max_offset) new_offset = max_offset;
 
-			vga.config.scan_len = new_offset;
+			if (CurMode->type == M_LIN24)
+				vga.config.scan_len = new_offset * 3u;
+			else
+				vga.config.scan_len = new_offset;
+
 			VGA_CheckScanLength();
 			break;
 
@@ -736,7 +764,10 @@ uint8_t VESA_SetDisplayStart(uint16_t x,uint16_t y,bool wait) {
 		panning_factor = 2; // this may be DOSBox specific
 		pixels_per_offset = 4;
 		break;
-	case M_LIN24: // FIXME
+	case M_LIN24:
+		pixels_per_offset = 8;
+		x *= 3;
+		break;
 	case M_LIN32:
 		pixels_per_offset = 2;
 		break;
